using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.IO;
using System.Diagnostics;
using System.Xml;

namespace ComplexNumbers
{
	public class Diagram
	{
		List<Complex> points = new List<Complex>();

		public int Count { get { return points.Count; } }

		public double GetX(int index)
		{
			return points[index].X;
		}

		public double GetY(int index)
		{
			return points[index].Y;
		}

		// Indexer = a special kind of property to select a value with an index
		public Complex this[int index] { get { return points[index]; } }

		public void Clear()
		{
			points.Clear();
		}

		public void AddPoint(double x, double y)
		{
			Complex p = new Complex(x, y);
			points.Add(p);
		}

		public void AddPoint(double x, double y, string name)
		{
			Complex p = new Complex(x, y, name);
			points.Add(p);
		}

		public void Inverse()
		{
			foreach (Complex p in points)
			{
				p.Invers();
			}
		}

		public void RemovePoint(int index)
		{
			points.RemoveAt(index);
		}

		public bool ReadFromXmlFile(string fileName)
		{
			XmlDocument xmlDoc = new XmlDocument();
			XmlNode node = null;
			bool success = true;
			try
			{
				xmlDoc.Load(fileName);
			}
			catch (Exception e)
			{
				Debug.WriteLine(e.Message);
				success = false;
				return success;
			}
			// Read content from the XML-Document
			points.Clear();
			// absolute path, starts with /
			node = xmlDoc.SelectSingleNode("/Diagram/Data");
			XmlNodeList nodeList;
			nodeList = node.SelectNodes("Point");	// relative path
			foreach (XmlNode pointNode in nodeList)
			{
				Complex p = new Complex();
				p.ReadFromXml(pointNode);
				points.Add(p);
			}

			return success;
		}

		public void SaveToXmlFile(string fileName)
		{
			XmlWriterSettings settings = new XmlWriterSettings();
			settings.Indent = true;
			settings.IndentChars = "\t";
			XmlWriter xw = XmlWriter.Create(fileName, settings);
			xw.WriteStartDocument();
			xw.WriteStartElement("Diagram");
			xw.WriteStartElement("Data");
			for (int i = 0; i < points.Count; i++)
			{
				points[i].WriteToXml(xw);
			}
			xw.WriteEndElement();	// </Points>
			xw.WriteEndElement();	// </Diagram>
			xw.Close();
		}

	}
}
