using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace ComplexNumbers
{
		public partial class FormMain : Form
		{
			double x = 0;	 // x as double value
			double y = 0;
			// current selected index of complex numbers collection
			// value of numeric up down control
			int index = 0;

			Diagram diagram = new Diagram();
			Plotter plotter;

			int pointNumber = 1; // to generate automaticlly a point name = number

			public FormMain()
			{
				InitializeComponent();
				plotter = new Plotter(panel);
				editX.Text = string.Format("{0:f2}", x);
				editY.Text = string.Format("{0:f2}", y);
				UpdateUI();
				this.panel.Paint += new System.Windows.Forms.PaintEventHandler(this.panel_Paint);
			}

			void GetPoint()
			{
				if (diagram.Count == 0) return;
				x = diagram.GetX(index);
				y = diagram.GetY(index);
				editX.Text = string.Format("{0:f2}", x);
				editY.Text = string.Format("{0:f2}", y);
			}

			void GetData()
			{
				// get all points from document (= diagram object) and 
				// update the user interface, table and diagram

				textOutput.Clear();
				for (int i = 0; i < diagram.Count; i++)
				{
					double x, y;
					x = diagram.GetX(i);
					y = diagram.GetY(i);
					textOutput.AppendText(string.Format("{0:d2}: {1,5:f2}  {2,5:f2}", i, x, y));
					if (diagram[i].Name != null)
						textOutput.AppendText(string.Format(" ({0:d})\r\n", diagram[i].Name));
					else
						textOutput.AppendText("\r\n");
				}
				UpdateUI();
				panel.Refresh();
			}

			void UpdateUI()
			{
				// Guarantees that the controls are 
				// enabled or disabled if necessary
				// and have adequate limits

				if (diagram.Count == 0)
				{
					numericUpDown.Enabled = false;
					numericUpDown.Maximum = 0;
					btnRemove.Enabled = false;
				}
				else
				{
					numericUpDown.Enabled = true;
					numericUpDown.Maximum = diagram.Count - 1;
					btnRemove.Enabled = true;
				}
			}
			
			void DrawDiagram()
			{
				double max = 2.0;
				plotter.DefineXAxis(-max, max, 0.5);
				plotter.DefineYAxis(-max, max, 0.5);
				plotter.DrawXAxis();
				plotter.DrawYAxis();
				for (int i = 0; i < diagram.Count; i++)
				{
					double x, y;
					x = diagram.GetX(i);
					y = diagram.GetY(i);
					string name = diagram[i].Name;
					plotter.Mark(Color.Blue, Color.Beige, x, y, 4);
					plotter.TextOut(x + 0.01*max, y + 0.01*max, name);
				}
			}

			private void btnAdd_Click(object sender, EventArgs e)
			{
				string name = pointNumber.ToString();
				pointNumber++;
				diagram.AddPoint(x, y, name);
				numericUpDown.Maximum++;
				numericUpDown.Value = diagram.Count - 1;
				GetData();
			}

			private void menuItemFileSave_Click(object sender, EventArgs e)
			{
				DialogResult result;
				result = saveFileDialog.ShowDialog();
				if (result == DialogResult.OK)
				{
					diagram.SaveToXmlFile(saveFileDialog.FileName);
				}
			}

			private void menuItemFileOpen_Click(object sender, EventArgs e)
			{
				OpenFileDialog dlg = new OpenFileDialog();
				dlg.Filter = "XML-Files|*.xml";
				DialogResult result = dlg.ShowDialog();
				if (result == DialogResult.OK)
				{
					bool success;
					success = diagram.ReadFromXmlFile(dlg.FileName);					
					if (success)
					{
						pointNumber = diagram.Count;
						GetData();	
					}
				}	
			}

			private void menuItemExit_Click(object sender, EventArgs e)
			{
				this.Close();
			}

			private void panel_Paint(object sender, PaintEventArgs e)
			{
				DrawDiagram();
			}

			private void editX_Validating(object sender, CancelEventArgs e)
			{
				double d;
				if (double.TryParse(editX.Text, out d))
				{
					x = d;
				}
				else
				{
					MessageBox.Show("Please enter a number!");
					e.Cancel = true;
				}
			}

			private void editY_Validating(object sender, CancelEventArgs e)
			{
				double d;
				if (double.TryParse(editY.Text, out d))
				{
					y = d;
				}
				else
				{
					MessageBox.Show("Please enter a number!");
					e.Cancel = true;
				}

			}

			private void btnRemove_Click(object sender, EventArgs e)
			{
				diagram.RemovePoint(index);
				GetData();
				GetPoint();
			}

			private void numericUpDown_ValueChanged(object sender, EventArgs e)
			{
				index = (int)numericUpDown.Value;
				GetPoint();
			}

			private void menuItemClear_Click(object sender, EventArgs e)
			{
				diagram.Clear();
				pointNumber = 1;
				GetData();
			}

			private void btnInvert_Click(object sender, EventArgs e)
			{
				diagram.Inverse();
				GetData();
			}

			private void menuItemAddLine_Click(object sender, EventArgs e)
			{
				double y;
				for (y = -2.0; y <= 2.0; y += 0.5)
				{
					string name = pointNumber.ToString();
					pointNumber++;
					diagram.AddPoint(1, y, name);
				}
				GetData();
			}
	 }
}